/*
    This file is part of RShape.

    RShape is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RShape is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with RShape.  If not, see <http://www.gnu.org/licenses/>.

   Copyright (C) 2008-2010 Neil Birkbeck
*/
package com.nacb.rshape;

import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;

/**
 * Circle representation.
 */
public class Circle extends Shape{
    Point2D center;
    double radius;

    /**
     * Construct a circle with unit diameter.
     */
    public Circle(){
	center = new Point2D(0.5, 0.5);
	radius = 0.5;
    }

    /**
     * Construct a circle with the given center and radius.
     *
     * @param c the center of the circle.
     * @param r the radius     
     */
    public Circle(Point2D c, double r){
	center = c;
	radius = r;
    }

    /**
     * Scale the circle.
     * @param s the amount to scale the circle by.
     */
    public void scale(double s){
	center = new Point2D(center.getX()*s,
				    center.getY()*s);
	radius *= s;
    }

    /**
     */
    public void add(Point2D p){
	center = new Point2D(center.getX() + p.getX(),
			     center.getY() + p.getY());
    }

    /**
     */
    public void subtract(Point2D p){
	center = new Point2D(center.getX() - p.getX(),
				    center.getY() - p.getY());
    }

    /**
     */
    public void center(){
	center = new Point2D(0,0);
    }

    /**
     */
    public void draw(Graphics2D g){
	AffineTransform affine = g.getTransform();

	GeneralPath path = new GeneralPath();
	
	double lastx = radius;
	double lasty = 0;

	g.translate(center.getX(), center.getY());
	
	path.moveTo((float)radius, 0.0f);

	double len = 0.4*2.0*Math.PI/16.0;

	for(int i=1; i<=16; i++){
	    double theta = ((double)i/16.0)*2.0*Math.PI;
	    double c = Math.cos(theta);
	    double s = Math.sin(theta);

	    path.curveTo((float)(lastx - lasty*len),
			 (float)(lasty + lastx*len),
			 (float)(c*radius + s*radius*len),
			 (float)(s*radius - c*radius*len),
			 (float)(c*radius), 
			 (float)(s*radius));
	    lastx = c*radius;
	    lasty = s*radius;
	}
	path.closePath();

	if(thick>0){
	    g.setStroke(new BasicStroke((float)(thick)));
	    g.setColor(lineColor);
	    g.draw(path);
	}

	g.setColor(color);
	g.fill(path);

	g.setTransform(affine);
    }

    /**
     */
    public String getData(){
	double p1x = center.getX() - radius;
	double p1y = center.getY();

	double p2x = center.getX() + radius;
	double p2y = center.getY();
	    
	
	return "M "+(center.getX() + radius)+" "+center.getY()+" A "+radius+" "+radius + " 0 1 1 "+ p1x + " " + p1y + " A " + radius + " " + radius + " 0 1 1 " + p2x + " " + p2y + " z";
    }
};