/*
    This file is part of RShape.

    RShape is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RShape is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with RShape.  If not, see <http://www.gnu.org/licenses/>.

   Copyright (C) 2008-2010 Neil Birkbeck
*/
package com.nacb.rshape;

import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.io.*;
import java.util.*;


public class ShapeTransferable implements Transferable{
    Vector<Shape> pgons;
    int cw, ch;
    public ShapeTransferable(Vector<Shape> pgons, int cw, int ch){
	this.pgons = pgons;
	this.cw = cw;
	this.ch = ch;
    }
    public ShapeTransferable(Shape pgon, int cw, int ch){
	pgons = new Vector<Shape>();
	pgons.addElement(pgon);

	this.cw = cw;
	this.ch = ch;
    }
    public Object getTransferData(DataFlavor flavor){
	System.out.println("Get transfer data " + flavor.getMimeType());
	String s;
	String w = String.valueOf(cw);//widthField.getText();
	String h = String.valueOf(ch);//heightField.getText();
	s = new String("<?xml version=\"1.0\" encoding=\"UTF-8\"?><svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" width=\""+ w +"\" height=\"" + h + "\" viewBox=\"0 0 "+ w + " " + h +"\" version=\"1.1\">");
	    
	s += "<g>";
	for(int i=0; i<pgons.size(); i++){
	    Shape pgon = pgons.elementAt(i);
	    String pdata = pgon.getData();
	    String pcol = pgon.getFill()?pgon.getSVGColor():"none";
	    String scol = pgon.getSVGLineColor();

	    s += "<path style=\"stroke: "+scol+"; stroke-width:"+pgon.thick+"; fill-rule: nonzero; fill: "+pcol+"; opacity: 1;\" d=\""+pdata+"\" />";
	    //transform=\"matrix("+pgon.scale+",0,0,"+pgon.scale+","+pgon.pos.x","+pgon.pos.y")\" />";
	}
	s += "</g>";
	s += "</svg>";
	//System.out.println("\n"+s+"\n");
	    
	if(flavor.getMimeType().contains("image/svg"))
	    return new ByteArrayInputStream(s.getBytes());
	else if(flavor.getMimeType().contains("text/plain") ||
		flavor.getMimeType().contains("x-moz") ||
		flavor.getMimeType().contains("uri-list") ||
		flavor.getMimeType().contains("application/x-java-file")){
		

	    String append = "";
	    if(flavor.getMimeType().contains("uri-list"))
		append = "\r\n";
		

	    try{
		File file = File.createTempFile("poly-",".svg");
		FileWriter writer = new FileWriter(file);
		writer.write(s);
		writer.close();
		    
		Vector<File> list = new Vector<File>();
		list.add(file);
		if(flavor.getMimeType().contains("application/x-java-file"))
		    return list;
		    

		return new ByteArrayInputStream((file.toURI().toString()+append).getBytes());
	    }
	    catch(Exception e){
		System.out.println("Caught exception "+e);
		return new ByteArrayInputStream((new String("file:///temp/nofile.svg")+append).getBytes());
	    }
	}
	return new ByteArrayInputStream(s.getBytes());
		
    }
	
    public DataFlavor[] 	getTransferDataFlavors(){
	DataFlavor [] flav = new DataFlavor[3];
	try {
		
	    //"application/x-java-file-list");//"text/uri-list");
	    //flav[2] = new DataFlavor("text/x-moz-file-promise");
	    //flav[1] = new DataFlavor("image/png");//image/jpg");
	    flav[2] = new DataFlavor("text/plain");//image/jpg");
	    flav[1] = DataFlavor.javaFileListFlavor;//new DataFlavor(Class.forName("java.util.List"),
	    flav[0] = new DataFlavor("image/svg+xml");//image/jpg");
	}
	catch(Exception e){
	    System.out.println("Caught exception "+e);
	}
	return flav;
    }
    public boolean 	isDataFlavorSupported(DataFlavor flavor){
	System.out.println("Data supported "+flavor.getMimeType());
	return false;
    }
};